#!/usr/bin/env python
from pytz import timezone
import requests
import sys
from time import sleep
from datetime import datetime
from Adafruit_CharLCD import Adafruit_CharLCD

lcd = Adafruit_CharLCD(rs=26, en=19, d4=13, d5=6, d6=5, d7=11, cols=16, lines=2)
lcd.clear()

import Adafruit_DHT as dht

# Yahoo Weather City ID's more available at https://weather.yahoo.com
cities = [  ('Amsterdam', '727232', 'Europe/Amsterdam'),
            ('Athens', '946738', 'Europe/Athens'),
            ('Bangkok',  '1225448', 'Asia/Bangkok'),
            ('Beijing', '2151330',  'Asia/Shanghai'),
            ('Berlin', '638242', 'Europe/Berlin'),
            ('Bern', '781788', 'Europe/Zurich'),
            ('Bratislava', '818717', 'Europe/Bratislava'),
            ('Brussels', '968019', 'Europe/Brussels'),
            ('Bucharest', '868274', 'Europe/Bucharest'),
            ('Budapest', '804365', 'Europe/Budapest'),
            ('Cape Town', '1591691', 'Africa/Johannesburg'),
            ('Chicago', '2379574', 'America/Chicago'),
            ('Copenhagen', '554890', 'Europe/Copenhagen'),
            ('Dhaka', '1915035',  'Asia/Dhaka'),
            ('Dublin', '560743', 'Europe/Dublin'),
            ('Hanoi', '91888417', 'Asia/Jakarta'),
            ('Helsinki', '565346', 'Europe/Helsinki'),
            ('Istanbul', '2344116', 'Europe/Istanbul'),
            ('Jakarta', '1047378', 'Asia/Jakarta'),
            ('Jerusalem', '1968222', 'Asia/Jerusalem'),
            ('Kiev', '924938', 'Europe/Kiev'),
            ('Kuantan', '1154799', 'Asia/Kuala_Lumpur'),
            ('Lima', '418440', 'America/Lima'),
            ('Lisbon', '742676', 'Europe/Lisbon'),
            ('Ljubljana', '530634', 'Europe/Ljubljana'),
            ('London',   '44418','Europe/London'),
            ('Los Angeles', '12795609', 'America/Los_Angeles'),
            ('Madrid', '766273', 'Europe/Madrid'),
            ('Manila', '1199477', 'Asia/Manila'),
            ('Melbourne', '1103816',  'Australia/Melbourne'),
            ('Mexico City', '116545', 'America/Mexico_City'),
            ('Montreal', '3534', 'America/Montreal'),
            ('Moscow', '2122265', 'Europe/Moscow'),
            ('Mumbai', '2295411', 'Asia/Calcutta'),
            ('New York', '2459115', 'America/New_York'),
            ('Oslo', '862592', 'Europe/Oslo'),
            ('Paris', '615702', 'Europe/Paris'),
            ('Prague', '796597', 'Europe/Prague'),
            ('Rio', '455825', 'America/Sao_Paulo'),
            ('Rome', '721943', 'Europe/Rome'),
            ('Santiago', '349859', 'America/Santiago'),
            ('Seoul', '1132599', 'Asia/Seoul'),
            ('Singapore', '1062617', 'Asia/Singapore'),
            ('Sofia', '839722', 'Europe/Sofia'),
            ('Stockholm', '906057', 'Europe/Stockholm'),
            ('Taipei', '2306179', 'Asia/Taipei'),
            ('Tokyo', '1118370', 'Asia/Tokyo'),
            ('Tucuman', '466865', 'America/Argentina/Tucuman'),
            ('Vienna', '551801', 'Europe/Vienna'),
            ('Warsaw', '523920', 'Europe/Warsaw')]

city_codes = ",".join([c[1] for c in cities])

def getWeatherConditions() :
    try :
        url = 'https://query.yahooapis.com/v1/public/yql'
        # encode query string for request
        query = ('select item.condition from weather.forecast where woeid in ('
                + city_codes + ')')
        query_strings = {'q': query, 'format' : 'json'}
        # headers to disable caching (in theory)
        headers = {'Pragma': 'no-cache', 'Cache-Control': 'no-cache'}
        # get weather data from Yahoo
        r = requests.get(url,
                         params=query_strings,
                         headers=headers,
                         timeout=30)
        # return data formatted to JSON
        return r.json(), r.status_code
    except requests.exceptions.Timeout:
        return "Timeout", 408
    except requests.exceptions.ConnectionError:
        return "Connection Error", 404
    except Exception:
        e = sys.exc_info()[0]
        print( "Error: {0}".format(e) )
        return "Undefined Error", 0
    

if __name__ == "__main__":
    while 1:
        data, code = getWeatherConditions()
        if code != 200:
            # Status Code Error
            print (data)
            if type(data) != type(str()):
                data = "Unknown"
            lcd.clear()
            lcd.message("ERROR {0}\n{1}".format(code, data))
            sleep(4)
        elif( type(data) != type(dict()) or "error" in data):
            # Feed Error
            print (data)
            lcd.clear()
            lcd.message("FEED\nERROR")
            sleep(4)
        else:
            count = data["query"]["count"]
            # Loop through cities
            for c in range(0, count):
                # Retrieve city name, condition and temperature
                city = cities[c][0]
                cond = data["query"]["results"]["channel"][c]["item"]["condition"]["text"]
                temp = data["query"]["results"]["channel"][c]["item"]["condition"]["temp"]
                lcd.clear()
                # Display city temperature and condition
                lcd.message(city + " " + temp)
                lcd.write8(223, True)
                lcd.message("F\n" + cond)
                sleep(2)

                # Display city temperature and local city time
                lcd.clear()
                lcd.message(city + " " + temp)
                lcd.write8(223, True)
                lcd.message("F\n")
                lcd.message(datetime.now(timezone(cities[c][2])).strftime('%I:%M %p %z'))

                sleep(2)
                
                # Poll DHT22
                h,t = dht.read_retry(dht.DHT22, 4)
                t = t * 9/5.0 + 32
                lcd.clear()
                # Display time
                lcd.message(datetime.now().strftime('%I:%M %p %b %d'))
                # Display temperature an humidity
                lcd.message("\n{0:0.1f}".format(t))
                lcd.write8(223, True)
                lcd.message("F   {0:0.1f}%".format(h))
                
                sleep(2)
